/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.crop.model;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;

/**
 * A CropDefinition contains the possible Varieties and Classifications for a
 * crop.
 *
 * @author Wytse Visser
 */
public class CropDefinition extends CropBean implements Serializable, PropertyChangeListener {
    
    public static final String EXT = "ctn";
    private Crop crop;
    public static final String PROP_VARIETIES = "varieties";
    private CropBeans<Variety> varieties;
    public static final String PROP_CLASSIFICATIONS = "classifications";
    private CropBeans<Classification> classifications;
    
    public CropDefinition(EnterpriseIdProvider eip, Crop crop) {
        super(eip, ItemIdType.CTN);
        setCrop(crop);
        varieties = new CropBeans(PROP_VARIETIES);
        classifications = new CropBeans(PROP_CLASSIFICATIONS);
    }
    
    public final void setCrop(Crop crop) {
        this.crop = crop;
        crop.addPropertyChangeListener(this);
    }
    
    public Crop getCrop() {
        return crop;
    }
    
    @Override
    public void setColor(Color color) {
        getCrop().setColor(color);
    }
    
    @Override
    public Color getColor() {
        return getCrop().getColor();
    }
    
    public void setClassifications(List<Classification> newClassifications) {
        classifications.setCropBeans(newClassifications);
    }
    
    public List<Classification> getClassifications() {
        return classifications.toList();
    }
    
    public void addClassification(Classification classification) {
        classifications.add(classification);
    }
    
    public void removeClassification(Classification classification) {
        classifications.remove(classification);
    }
    
    public void setVarieties(List<Variety> newVarieties) {
        varieties.setCropBeans(newVarieties);
    }
    
    public List<Variety> getVarieties() {
        return varieties.toList();
    }
    
    public void addVariety(Variety variety) {
        varieties.add(variety);
    }
    
    public void removeVariety(Variety variety) {
        varieties.remove(variety);
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        getPropertyChangeSupport().firePropertyChange(evt);
    }
    
    @Override
    public String toString() {
        return getName();
    }
    
    @Override
    public String getName() {
        return getCrop().getName();
    }
    
    @Override
    public void setName(String name) {
        getCrop().setName(name);
    }
    
    @Override
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        super.addPropertyChangeListener(listener);
        varieties.addPropertyChangeListener(listener);
        classifications.addPropertyChangeListener(listener);
    }

    /**
     * This class provides functionality for managing a collection of CropBeans.
     * It automatically fires a PropertyChangeEvent if its contents change.
     *
     * @param <T> The type of CropBean. Can be either a Variety or a
     * Classification.
     */
    private class CropBeans<T extends CropBean> implements Serializable {
        
        private final String propertyName;
        private List<T> cropBeans;
        
        public CropBeans(String propertyName) {
            this.propertyName = propertyName;
            cropBeans = new ArrayList<>();
        }
        
        void add(T cropBean) {
            List<T> old = cropBeans;
            cropBeans.add(cropBean);
            getPropertyChangeSupport().firePropertyChange(propertyName, null, cropBean);
            fireUndoableEvent(propertyName, CropDefinition.this, old, cropBeans);
        }
        
        void remove(T cropBean) {
            List<T> old = cropBeans;
            cropBeans.remove(cropBean);
            getPropertyChangeSupport().firePropertyChange(propertyName, cropBean, null);
            fireUndoableEvent(propertyName, CropDefinition.this, old, cropBeans);
        }
        
        void setCropBeans(List<T> cropBeans) {
            List<T> old = new ArrayList<>(cropBeans);
            this.cropBeans = cropBeans;
            getPropertyChangeSupport().firePropertyChange(propertyName, old, cropBeans);
        }
        
        List<T> toList() {
            return cropBeans;
        }
        
        void addPropertyChangeListener(PropertyChangeListener listener) {
            for (CropBean cropBean : cropBeans) {
                cropBean.addPropertyChangeListener(listener);
            }
        }
    }
}
